#!/bin/sh

if [ "$#" -ne 3 ]; then
    echo "Usage: $0 <on/off> <anon_folder> <priv_img_url>"
    exit 1
fi

ACTION="$1"
ANON_FOLDER="$2"
PRIVACY_IMAGE_URL="$3"

# Files will be placed here at installation
APACHE_TEMPLATE="$ANON_FOLDER/privacy_image.conf.template"
SERVICE_TEMPLATE="$ANON_FOLDER/privacy.service.template"

# Put these somewhere persistent
PERSIST="/usr/local/anon"
TBACK="$PERSIST/transfer.backup"
BLOCKER_FILE="$PERSIST/blocker.sh"
EMPTY_FILE="$PERSIST/empty"
IP_TABLES_FILE="$PERSIST/iptables"
ACTION_OVERRIDE_FOLDER="$PERSIST/action.override"
LOG="$PERSIST/run_log.txt"

# Firmware dependent settings
TFILE="/etc/apache2/transfer/transfer.conf"
VHOST="/etc/apache2/conf.d/vhosts/all"
MCAST="/usr/sbin/mcast-always-setter"
SERVICE_DIR="/etc/systemd/system"
SNAPSHOT="/usr/sbin/jpeg_snapshot"
EVENT_DIR="/etc/actionengine/user/"
RTSP_CONF="/etc/rtsp/rtsp.conf"
ACTION_TEMPLATES="/usr/share/actionengine/user/actiontemplates"

APACHE_FILE="$VHOST/privacy_mode.conf"
SERVICE_FILE="$SERVICE_DIR/privacy.service"

RTSP_PORT=$(cat $RTSP_CONF | grep Port | grep -o "[0-9]*")
if [ -z "$RTSP_PORT" ]; then
    echo "Can't locate RTSP port"
    exit 1
fi
echo "Using RTSP-port " $RTSP_PORT

CHAIN_1="INPUT -p tcp --dport $RTSP_PORT -j REJECT"
CHAIN_2="INPUT -p udp --dport $RTSP_PORT -j REJECT"
CHAIN_3="OUTPUT -p tcp --dport $RTSP_PORT -j REJECT"
CHAIN_4="OUTPUT -p udp --dport $RTSP_PORT -j REJECT"

mkdir -p $PERSIST
echo "$(date) : Running action $ACTION" >> $LOG

case "$ACTION" in
    "on")

        # Block the http streams
        if [ ! -f $TBACK ]; then
            cp $TFILE $TBACK
        fi
        sed $TBACK -e '/Transfer.*monolith/s/^/#/' > $TFILE

        sed $APACHE_TEMPLATE \
            -e "s/__PRIVACY_IMAGE__/$(echo $PRIVACY_IMAGE_URL | sed 's/\//\\\//g')/" \
            > $APACHE_FILE

        # Block rtsp streams
        chattr +i $RTSP_CONF # Prevent user from changing rtsp port

        if ! iptables -C $CHAIN_1 2> /dev/null; then
            iptables -A $CHAIN_1
        fi
        if ! iptables -C $CHAIN_2 2> /dev/null; then
            iptables -A $CHAIN_2
        fi
        if ! iptables -C $CHAIN_3 2> /dev/null; then
            iptables -A $CHAIN_3
        fi
        if ! iptables -C $CHAIN_4 2> /dev/null; then
            iptables -A $CHAIN_4
        fi
        iptables-save > $IP_TABLES_FILE

        if ! ip6tables -C $CHAIN_1 2> /dev/null; then
            iptables-save | sed -e 's/icmp/icmp6/g' | ip6tables-restore -c
        fi
        ip6tables-save > ${IP_TABLES_FILE}.ipv6

        # Prevent multicast-always
        if ! mount | grep -w ${MCAST%/}; then
            echo "#!/bin/sh" > $BLOCKER_FILE
            chmod +x $BLOCKER_FILE
            mount --bind $BLOCKER_FILE $MCAST
        fi

        # Prevent snapshot in server report
        if ! mount | grep -w ${SNAPSHOT%/}; then
            touch $EMPTY_FILE
            mount --bind $EMPTY_FILE $SNAPSHOT
        fi

        # Prevent events from sending image or video
        $ANON_FOLDER/filter_events.sh on "$ACTION_TEMPLATES" "$ACTION_OVERRIDE_FOLDER"
        # These are probably not used, but just to make sure.
        rm $EVENT_DIR/rules/* $EVENT_DIR/configurations/* 2> /dev/null | true
        chown root:root "$EVENT_DIR/rules" "$EVENT_DIR/configurations"

        # Run this stuf only first time, not when the script calls the service
        if ! systemctl list-unit-files | grep privacy | grep enabled; then
            echo " -- FIRST INIT -- " >> $LOG

            # Prevent storage on sd-card or network storage
            systemctl mask dbus-com.axis.Storage.service
            systemctl mask storage-manager
            systemctl stop dbus-com.axis.Storage.service
            systemctl stop storage-manager

            # Add start up service
            # bind mounts are cleared on reboot
            # iptables are cleared on reboot
            sed $SERVICE_TEMPLATE \
                -e "s/__SCRIPT_PATH__/$(echo $0 | sed 's/\//\\\//g')/g" \
                -e "s/__ANON_PATH__/$(echo $ANON_FOLDER | sed 's/\//\\\//g')/g" \
                -e "s/__PRIVACY_IMAGE_URL__/$(echo $PRIVACY_IMAGE_URL | sed 's/\//\\\//g')/g" \
                > $SERVICE_FILE
            systemctl daemon-reload
            systemctl enable privacy

            # Restart the service
            systemctl restart action-engine-user.service
            systemctl restart httpd

        fi


        ;;
    "off")

        # FIXME: Remove blocker file and empty file

        # Allow http streams
        if ! mv $TBACK $TFILE 2> /dev/null; then
            echo "NOTICE: No backup of transfer.conf found, applying filter"
            sed $TFILE -i -e '/Transfer.*monolith/s/^#*//'
        fi
        rm $APACHE_FILE 2> /dev/null | true
        systemctl restart httpd

        # Enable rtps streams
        chattr -i $RTSP_CONF
        systemctl disable privacy
        iptables -D $CHAIN_1 2> /dev/null | true
        iptables -D $CHAIN_2 2> /dev/null | true
        iptables -D $CHAIN_3 2> /dev/null | true
        iptables -D $CHAIN_4 2> /dev/null | true
        ip6tables -D $CHAIN_1 2> /dev/null | true
        ip6tables -D $CHAIN_2 2> /dev/null | true
        ip6tables -D $CHAIN_3 2> /dev/null | true
        ip6tables -D $CHAIN_4 2> /dev/null | true

        # Enable multicast-always
        while umount $MCAST 2> /dev/null; do
            :
        done
        rm $BLOCKER_FILE 2> /dev/null | true

        # Enable server report image
        while umount $SNAPSHOT 2> /dev/null; do
            :
        done
        rm $EMPTY_FILE 2> /dev/null | true

        # Enable storage
        systemctl unmask dbus-com.axis.Storage.service
        systemctl unmask storage-manager
        systemctl daemon-reload
        systemctl start dbus-com.axis.Storage.service
        systemctl start storage-manager

        # Enable events
        $ANON_FOLDER/filter_events.sh off "$ACTION_TEMPLATES" "$ACTION_OVERRIDE_FOLDER"
        chown actionengined:actionengined "$EVENT_DIR/rules" "$EVENT_DIR/configurations"
        
        ;;
    *)

        echo "Action should be 'on' or 'off'"
        exit 1

        ;;
esac
